@@ -1,21 +1,14 @@
-# $Id: TLTREE.pm 44232 2017-05-06 23:06:56Z karl $
+# $Id: TLTREE.pm 55106 2020-05-11 21:11:50Z karl $
 # TeXLive::TLTREE.pm - work with the tree of all files
-# Copyright 2007-2017 Norbert Preining
+# Copyright 2007-2020 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLTREE;
 
-my $svnrev = '$Revision: 44232 $';
-my $_modulerevision;
-if ($svnrev =~ m/: ([0-9]+) /) {
-  $_modulerevision = $1;
-} else {
-  $_modulerevision = "unknown";
-}
-sub module_revision {
-  return $_modulerevision;
-}
+my $svnrev = '$Revision: 55106 $';
+my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
+sub module_revision { return $_modulerevision; }
 
 use TeXLive::TLUtils;
 
@@ -78,9 +71,10 @@ sub init_from_git {
   my $svnroot = $self->{'svnroot'};
   my $retval = $?;
   my %files;
+  my %deletedfiles;
   my @lines;
 
-  my @foo = `cd $svnroot; git log --pretty=format:COMMIT=%h --name-only`;
+  my @foo = `cd $svnroot; git log --pretty=format:COMMIT=%h --no-renames --name-status`;
   if ($retval != 0) {
     $retval /= 256 if $retval > 0;
     tldie("TLTree: git log in $svnroot returned $retval, stopping.\n");
@@ -88,6 +82,7 @@ sub init_from_git {
   chomp(@foo);
 
   my $curcom = "";
+  my $rev = 0;
   for my $l (@foo) {
     if ($l eq "") {
       $curcom = "";
@@ -97,17 +92,38 @@ sub init_from_git {
       $rev++;
       next;
     } else {
-      # we only use the first occurrence of $f from the top,
-      # that is the most recent change
-      $files{$l} = $rev if (not(defined($files{$l})));
+      # output is 
+      #   STATUS FILENAME
+      # where STATUS is as follows:
+      #   Added (A), Copied (C), Deleted (D), Modified (M), Renamed (R), have their type (i.e. regular file,
+      #   symlink, submodule, ...) changed (T), are Unmerged (U), are Unknown (X), or have had their pairing Broken (B).
+      if ($l =~ m/^(A|C|D|M|R|T|U|X|B)\S*\s+(.*)$/) {
+        my $status = $1;
+        my $curfile = $2;
+        #
+        # check whether the file was already removed
+        if (!defined($files{$curfile}) && !defined($deletedfiles{$curfile})) {
+          # first occurrence of that file
+          if ($status eq "D") {
+            $deletedfiles{$curfile} = 1;
+          } else {
+            $files{$curfile} = $rev;
+          }
+        }
+      } else {
+        print STDERR "Unknown line in git output: >>$l<<\n";
+      }
     }
   }
 
   # now reverse the order
   for my $f (keys %files) {
     my $n = - ( $files{$f} - $rev ) + 1;
+    # special case for TL: remove Master if it is present
+    $f =~ s!^Master/!!;
     push @lines, "             $n $n dummy $f"
   }
+  # debug(join("\n", @lines));
   # TODO needs to be made better!
   $self->{'revision'} = $rev;
   $self->_initialize_lines(@lines);
@@ -183,8 +199,9 @@ sub _initialize_lines {
       next if ($1 eq "D"); # ignore files which are removed
       next if -d $entry && ! -l $entry; # keep symlinks to dirs (bin/*/man),
                                         # ignore normal dirs.
-      # collect architectures, assuming nothing is in bin/ but arch subdirs.
-      if ($entry =~ m,^bin/([^/]*)/,) {
+      # collect architectures; bin/ has arch subdirs plus the plain man
+      # special case.
+      if ($entry =~ m,^bin/([^/]*)/, && $entry ne "bin/man") {
         $archs{$1} = 1;
       }
       $self->{'_allfiles'}{$entry}{'lastchangedrev'} = $lastchanged;
@@ -307,13 +324,21 @@ of bin patterns for that arch).
 sub get_matching_files {
   my ($self, $type, $p, $pkg, $arch) = @_;
   my $ARCH = $arch;
-  my $PKGNAME = $pkg;
   my $newp;
-  eval "\$newp = \"$p\"";
-  if (!defined($newp)) {
-    print "Huuu: cannot generate newp from p: p=$p, pkg=$pkg, arch=$arch, type=$type\n";
+  {
+    my $warnstr = "";
+    local $SIG{__WARN__} = sub { $warnstr = $_[0]; };
+    eval "\$newp = \"$p\"";
+    if (!defined($newp)) {
+      die "cannot set newp from p: p=$p, pkg=$pkg, arch=$arch, type=$type";
+    }
+    if ($warnstr) {
+      tlwarn("Warning `$warnstr' while evaluating: $p "
+             . "(pkg=$pkg, arch=$arch, type=$type), returning empty list\n");
+      return ();
+    }
   }
-  return($self->_get_matching_files($type,$newp));
+  return $self->_get_matching_files($type,$newp);
 }
 
   
@@ -330,7 +355,7 @@ sub _get_matching_files {
   } elsif ($pattype eq "d") {
     @matchfiles = $self->files_under_path($patdata);
   } else {
-    die "Unknown pattern pattern type `$pattype' in $p";
+    die "Unknown pattern type `$pattype' in $p";
   }
   ddebug("p=$p; matchfiles=@matchfiles\n");
   return @matchfiles;
@@ -361,9 +386,9 @@ sub _get_files_matching_glob_pattern
 
   my @candfiles = @{$self->{'_filesofdir'}{$dirpart}};
   for my $f (@candfiles) {
-    ddebug("matching $f in $dirpart via glob $globline\n");
+    dddebug("matching $f in $dirpart via glob $globline\n");
     if ($f =~ /^$basepart$/) {
-      ddebug("hit: globline=$globline, $dirpart/$f\n");
+      dddebug("hit: globline=$globline, $dirpart/$f\n");
       if ("$dirpart" eq ".") {
         push @returnfiles, "$f";
       } else {
